package org.example.music.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.example.music.common.R;
import org.example.music.model.domain.Consumer;
import org.example.music.model.domain.ResetPasswordRequest;
import org.example.music.model.request.ConsumerRequest;
import org.example.music.service.ConsumerService;
import org.example.music.service.impl.ConsumerServiceImpl;
import org.example.music.service.impl.SimpleOrderManager;
import org.example.music.util.RandomUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpSession;
import java.util.concurrent.TimeUnit;

@Api(value = "ConsumerController", tags = {"用户控制类"})
@RestController
public class ConsumerController {

    @Autowired
    private ConsumerService consumerService;

    @Autowired
    ConsumerServiceImpl consumerServiceimpl;

    @Autowired
    private SimpleOrderManager simpleOrderManager;

    @Autowired
    StringRedisTemplate stringRedisTemplate;

    /**
     * TODO 前台页面调用 注册
     * 用户注册
     */
    @ApiOperation(value = "注册")
    @PostMapping("/user/add")
    public R addUser(@RequestBody ConsumerRequest registryRequest) {
        return consumerService.addUser(registryRequest);
    }

    /**
     * TODO 前台页面调用  登录
     * 登录判断
     */
    @ApiOperation(value = "登录")
    @PostMapping("/user/login/status")
    public R loginStatus(@RequestBody ConsumerRequest loginRequest, HttpSession session) {
        return consumerService.loginStatus(loginRequest, session);
    }

    /**
     * email登录
     */
    @ApiOperation(value = "email登录")
    @PostMapping("/user/email/status")
    public R loginEmailStatus(@RequestBody ConsumerRequest loginRequest, HttpSession session) {
        return consumerService.loginEmailStatus(loginRequest, session);
    }

    /**
     * 密码恢复（忘记密码）
     */
    @ApiOperation(value = "重置密码")
    @PostMapping("/user/resetPassword")
    public R resetPassword(@RequestBody ResetPasswordRequest passwordRequest) {
        Consumer user = consumerService.findByEmail(passwordRequest.getEmail());
        String code = stringRedisTemplate.opsForValue().get("code");
        if (user==null){
            return R.fatal("用户不存在");
        }else if (!code.equals(passwordRequest.getCode())){
            return R.fatal("验证码不存在或失效");
        }
        ConsumerRequest consumerRequest = new ConsumerRequest();
        BeanUtils.copyProperties(user, consumerRequest);
        System.out.println(user);
        System.out.println(consumerRequest);
        consumerRequest.setPassword(passwordRequest.getPassword());
        consumerServiceimpl.updatePassword01(consumerRequest);

        return R.success("密码修改成功");
    }

    /**
     * 发送验证码功能
     */
    @ApiOperation(value = "发送验证码")
    @GetMapping("/user/sendVerificationCode")
    public R sendCode(@RequestParam String email) {
        Consumer user = consumerService.findByEmail(email);
        if (user == null) {
            return R.fatal("用户不存在");
        }
        String code = RandomUtils.code();
        simpleOrderManager.sendCode(code,email);
        //保存在redis中
        stringRedisTemplate.opsForValue().set("code",code,5, TimeUnit.MINUTES);
        return R.success("发送成功");
    }


    /**
     * TODO 管理界面调用
     * 返回所有用户
     */
    @ApiOperation(value = "查询所有用户")
    @GetMapping("/user")
    public R allUser() {
        return consumerService.allUser();
    }


    /**
     * TODO 用户界面调用
     * 返回指定 ID 的用户
     */
    @ApiOperation(value = "用户详情")
    @GetMapping("/user/detail")
    public R userOfId(@RequestParam int id) {
        return consumerService.userOfId(id);
    }

    /**
     * TODO 管理界面的调用
     * 删除用户
     */
    @ApiOperation(value = "删除用户")
    @GetMapping("/user/delete")
    public R deleteUser(@RequestParam int id) {
        return consumerService.deleteUser(id);
    }

    /**
     * TODO 前后台界面的调用
     * 更新用户信息
     */
    @ApiOperation(value = "更新用户信息")
    @PostMapping("/user/update")
    public R updateUserMsg(@RequestBody ConsumerRequest updateRequest) {
        return consumerService.updateUserMsg(updateRequest);
    }

    /**
     * TODO 前后台更新用户的密码
     * 更新用户密码
     */
    @ApiOperation(value = "更新用户密码")
    @PostMapping("/user/updatePassword")
    public R updatePassword(@RequestBody ConsumerRequest updatePasswordRequest) {
        return consumerService.updatePassword(updatePasswordRequest);
    }

    /**
     * 更新用户头像
     */
    @ApiOperation(value = "更新用户头像")
    @PostMapping("/user/avatar/update")
    public R updateUserPic(@RequestParam("file") MultipartFile avatorFile, @RequestParam("id") int id) {
        return consumerService.updateUserAvator(avatorFile, id);
    }

}
